import base64
import base64
def image_byte_to_base64(byte_str):
    if not isinstance(byte_str, bytes):
        raise TypeError("expecte bytes , not {}".format(type(byte_str)))
    return base64.b64encode(byte_str)

def image_base64_to_byte(b64_str):
    image_data = base64.b64decode(b64_str)
    return image_data


import os

def progress_int_read(progress_path, default=-1):
    if not os.path.exists(progress_path):
        return default
    return int(open(progress_path, 'r').read())

def progress_int_write(progress_path, value_int):
    with open(progress_path, 'w', encoding='utf-8') as f:
        f.write("{}".format(value_int))


import datetime
import time
from contextlib import contextmanager
import logging
import functools
# from async_generator import asynccontextmanager

# https://stackoverflow.com/questions/37433157/asynchronous-context-manager


@contextmanager
def util_logger_context(fn_logger, prefix='test', width=80, fill=" "):
    try:
        now = time.time()
        yield
    finally:
        end = time.time()
        fn_logger('{}: cost {}s'.format(prefix, (end - now)).center(width, fill))


@contextmanager
def util_logger_statment_context(logger, msg='', method='info', width=80, fill="=", add_new_line=True):
    getattr(logger, method)("start {}".format(msg).center(width, fill))
    if add_new_line:
        getattr(logger, method)("")
    try:
        yield
    finally:
        if add_new_line:
            getattr(logger, method)("")
        getattr(logger, method)("end {}".format(msg).center(width, fill))


def prefix_concat(prefix1, prefix2):
    def prefix_to_str(prefix):
        if callable(prefix):
            return prefix()
        return prefix

    def _prefix():
        str1 = prefix_to_str(prefix1)
        str2 = prefix_to_str(prefix2)
        try:
            return str1 + str2
        except TypeError as e:
            raise TypeError("str1:{}, str2:{}, {}, prefix1:{}, prefix2:{}, callable:{}".format(str1, str2, e, prefix1, prefix2, callable(prefix2)))

        # return prefix_to_str(prefix1) + prefix_to_str(prefix2)
    return _prefix


class LoggerMultiLineLoggerAdapter(logging.LoggerAdapter):
    def __init__(self, prefix, logger, enable=None):
        # super(LoggerAdapter, self).__init__(logger, {})
        if isinstance(prefix, (str, bytes)):
            def _new_prefix(val):
                # 没有这个val的话，会造成无线递归
                return val
            prefix = functools.partial(_new_prefix, prefix)
        local_prefix = ''
        # enable = True
        if isinstance(logger, LoggerMultiLineLoggerAdapter):
            local_prefix, logger, enable = logger._prefix, logger.logger, logger._enable
        local_prefix = prefix_concat(local_prefix, prefix)
        if enable is None:
            enable = True
        # return cls(prefix, logger, enable=enable)
        super().__init__(logger, {})
        self._prefix = local_prefix
        self._enable = enable

    def get_prefix(self):
        if isinstance(self._prefix, str):
            return self._prefix
        if callable(self._prefix):
            return self._prefix()
        raise TypeError("unsupported prefix error  {}".format(type(self._prefix)))

    def process(self, msg, kwargs):
        lines = msg.splitlines()
        lines = ['[%s] %s' % (self.get_prefix(), e) for e in lines]
        return lines, kwargs
        # return '[%s] %s' % (self.prefix, msg), kwargs

    def log(self, level, msg, *args, **kwargs):
        """
        Delegate a log call to the underlying logger, after adding
        contextual information from this adapter instance.
        """
        if self.isEnabledFor(level) and self._enable:
            msg, kwargs = self.process(msg, kwargs)
            if isinstance(msg, (list, tuple)):
                pass
            else:
                msg = [msg]
            for line in msg:
                self.logger.log(level, line, *args, **kwargs)

    @classmethod
    def add_datetime_to_logger(cls, logger, date_format, prefix=''):
        def prefix_func():
            return '{}{}'.format(prefix, datetime.datetime.now().strftime(date_format))
        return cls(prefix_func, logger)

    @classmethod
    def create_from_logger(cls, prefix, logger):
        return cls(prefix, logger)
        # local_prefix = ''
        # enable = True
        # if isinstance(logger, cls):
        #     local_prefix, logger, enable = logger._prefix, logger.logger, logger._enable
        # local_prefix += prefix
        # return cls(prefix, logger, enable=enable)




from honcho.manager import Manager
from honcho.process import Process

class AllStopManager(Manager):

    def loop(self):
        """
        Start all the added processes and multiplex their output onto the bound
        printer (which by default will print to STDOUT).

        If one process terminates, all the others will be terminated by
        Honcho, and :func:`~honcho.manager.Manager.loop` will return.

        This method will block until all the processes have terminated.
        """
        def _terminate(signum, frame):
            self._system_print("%s received\n" % SIGNALS[signum]['name'])
            self.returncode = SIGNALS[signum]['rc']
            self.terminate()
        import signal
        from multiprocessing.queues import Empty
        import datetime
        KILL_WAIT = 5
        signal.signal(signal.SIGTERM, _terminate)
        signal.signal(signal.SIGINT, _terminate)


        self._start()

        exit = False
        exit_start = None

        while 1:
            try:
                msg = self.events.get(timeout=0.1)
            except Empty:
                if exit:
                    break
            else:
                if msg.type == 'line':
                    self._printer.write(msg)
                elif msg.type == 'start':
                    self._processes[msg.name]['pid'] = msg.data['pid']
                    self._system_print("%s started (pid=%s)\n"
                                       % (msg.name, msg.data['pid']))
                elif msg.type == 'stop':
                    self._processes[msg.name]['returncode'] = msg.data['returncode']
                    self._system_print("%s stopped (rc=%s)\n"
                                       % (msg.name, msg.data['returncode']))
                    if self.returncode is None:
                        self.returncode = msg.data['returncode']

            if self._all_started() and self._all_stopped():
                exit = True

            # if exit_start is None and self._all_started():
            #     # if self._any_stopped() and self.exit_on_any_stop:
            #     #     exit_start = self._env.now()
            #     #     self.terminate()
            #     if self._all_stopped():
            #         exit_start = self._env.now()
            #         self.terminate()

            # if exit_start is not None:
            #     # If we've been in this loop for more than KILL_WAIT seconds,
            #     # it's time to kill all remaining children.
            #     waiting = self._env.now() - exit_start
            #     if waiting > datetime.timedelta(seconds=KILL_WAIT):
            #         self.kill()
